import { VNode, HTMLAttributes, AllowedComponentProps, ComponentCustomProps, VNodeProps, Fragment, Teleport, Suspense, Component, DefineComponent, ComponentOptionsMixin, EmitsOptions, ExtractPropTypes, PropType } from 'vue';
import { SystemProps, ResponsiveValue, StyleProps, SystemStyleObject } from '@chakra-ui/styled-system';
import { Dict } from '@chakra-ui/utils';
import { CSSObject } from '@emotion/css';
import { DOMElements } from './system.utils.js';
import { IntrinsicElementAttributes } from './dom.types.js';
import { FunctionInterpolation } from '@emotion/serialize';

/**
 * Export component with custom type
 *
 * @example
 * export const CBox = CBoxImpl as ComponentWithProps<{ hello?: string }>
 */
type ComponentWithProps2<P> = {
    new (): {
        $props: AllowedComponentProps & ComponentCustomProps & VNodeProps & P & {
            [key: string]: any;
        };
    };
};
type AsPolymorphicProp = {
    as?: DOMElements | string | object;
};
type Tag = string | typeof Fragment | typeof Teleport | typeof Suspense | Component;
interface ChakraProps extends SystemProps, StyleResolverProps {
    /**
     * apply layer styles defined in `theme.layerStyles`
     */
    layerStyle?: string;
    /**
     * apply typography styles defined in `theme.textStyles`
     */
    textStyle?: string;
    /**
     * Reference styles from any component or key in the theme.
     *
     * @example
     * ```html
     * <c-box apply="styles.h3">This is a div</c-box>
     * ```
     *
     * This will apply styles defined in `theme.styles.h3`
     */
    apply?: string;
    /**
     * if `true`, it'll render an ellipsis when the text exceeds the width of the viewport or maxWidth set.
     */
    isTruncated?: boolean;
    /**
     * Used to truncate text at a specific number of lines
     */
    noOfLines?: ResponsiveValue<number>;
    /**
     * Internal prop used to label Chakra factory component tags
     */
    label?: string;
    /**
     * Internal prop used to label JSX component tags
     */
    __label?: string;
}
type ElementType<P = any> = {
    [K in keyof IntrinsicElementAttributes]: P extends IntrinsicElementAttributes[K] ? K : never;
}[keyof IntrinsicElementAttributes] | Component<P>;
type As<Props = any> = ElementType<Props>;
/**
 * Extract the props of a Vue element or component
 */
type PropsOf<T extends As> = T & {
    as?: As;
};
type HTMLChakraProps<T extends As> = Omit<PropsOf<T>, T extends "svg" ? "ref" | "children" | keyof StyleProps : "ref" | keyof StyleProps> & ChakraProps & {
    as?: As;
};
declare global {
    namespace h.JSX {
        interface Element extends VNode {
        }
        interface ElementClass {
            $props: {};
        }
        interface ElementAttributesProperty {
            $props: {};
        }
        interface IntrinsicAttributes extends Omit<HTMLAttributes, "color">, ChakraProps {
        }
    }
}

interface BaseStyleResolverProps {
    as?: ChakraTagOrComponent;
    __css?: SystemStyleObject;
    sx?: SystemStyleObject;
    css?: CSSObject;
    noOfLines?: ResponsiveValue<number>;
    isTruncated?: boolean;
    apply?: ResponsiveValue<string>;
    componentName?: String;
    label?: string;
    baseStyle?: SystemStyleObject | ((props: StyleResolverProps) => SystemStyleObject);
    /**
     * User provided styles from component/chakra API
     */
    styles?: SystemStyleObject;
    /**
     * This attribute/property is reserved for all TSX component definitions.
     * It is referenced by the chakra factiry function to
     * preserve the component's label class
     */
    __label?: string;
    /**
     * @internal prop to handle factory components as raw elements
     */
    __chakraIsRaw?: boolean;
    theme?: Dict<any>;
    styleConfig?: Dict<any>;
}
interface StyleResolverProps extends BaseStyleResolverProps, SystemProps {
}
interface StyleResolverOptions extends StyleResolverProps {
    truncateStyle?: CSSObject;
    theme?: any;
}
declare const chakraProps: {
    as: PropType<ChakraTagOrComponent>;
    __css: PropType<SystemStyleObject | undefined>;
    sx: PropType<SystemStyleObject | undefined>;
    css: PropType<CSSObject | undefined>;
    noOfLines: PropType<ResponsiveValue<number> | undefined>;
    baseStyle: PropType<SystemStyleObject | ((props: StyleResolverProps) => SystemStyleObject) | undefined>;
    isTruncated: PropType<boolean | undefined>;
    layerStyle: PropType<ResponsiveValue<string & {}> | undefined>;
    textStyle: PropType<ResponsiveValue<string & {}> | undefined>;
    apply: PropType<ResponsiveValue<string> | undefined>;
    label: PropType<string | undefined>;
    modelValue: PropType<string | boolean | object>;
    /**
     * @warning
     * @internal
     * This internal is an internal ChakraFactoryFunction prop that
     * is used to determine how events are handled on Chakra Factory
     * components.
     *
     * For example, if a factory component is considered to be raw (i.e. `__chakraIsRaw: true`),
     * then, we do not pass v-model event listeners onto the component. This means that
     * `v-model` will not work in the template context.
     *
     * You can see how this prop is used in the `c-input` component.
     *
     * THIS PROP IS A NON-DOCUMENTED PROP, AND IS ONLY TO BE USED FOR INTERNAL DEVELOPMENT.
     */
    __chakraIsRaw: PropType<boolean>;
};
type ChakraBaseComponentProps = typeof chakraProps;
type ChakraTagOrComponent = DOMElements | Component | "router-link" | "nuxt-link";
type ChakraFactoryComponent<Props extends {} = {}> = DefineComponent<Props, () => JSX.Element, {}, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, EmitsOptions, string, VNodeProps & AllowedComponentProps & ComponentCustomProps, Readonly<ExtractPropTypes<Props>>, {}>;
type ComponentWithProps<Props extends {}> = ChakraFactoryComponent<Props>;
declare function ___chakra___(tag: ChakraTagOrComponent, options?: StyledOptions): DefineComponent<{}, () => JSX.Element, {}, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, EmitsOptions, string, VNodeProps & AllowedComponentProps & ComponentCustomProps, Readonly<ExtractPropTypes<{}>>, {}>;
/**
 * Chakra factory serves as an object of chakra enabled HTML elements,
 * and also a function that can be used to enable custom component receive chakra's style props.
 * @param tag Tag or Component
 * @param options resolver options
 *
 * How does it work?
 *
 * 1. Components returned from the chakra factory can be styled after consuming them
 *    @example
 *    ```js
 *    const Form = chakra('form') // returns a VNode you can use in the template directly
 *    ```
 *
 * 2. Chakra components can directly be styled upon creation using the options object of type `StyleResolverProps`
 *    This resolves style object for component styles defined in the theme.
 *
 *    Styling components using the chakra factory function can be done using the following keys from the theme:
 *    - `baseStyle`
 *    - `layerStyle`
 *    - `textStyle`
 *
 *    @example
 *    ```js
 *    const MyCustomButton = chakra('button', {
 *     baseStyle: {
         bg: 'papayawhip,
         color: 'red.500,
         px: 4,
         py: 3
       }
 *    })
 *    ```
 *    ```html
 *    <my-custom-button>Hello Papaya Button</my-custom-button>
 *    ```
 *
 *    See more about the style resolution in the `resolveStyles` function.
 *
 * 3. Chakra components created and styled using the `chakra` factory can be overriden in the template by applying
 *    style properties directly
 *
 *    @example
 *    ```html
 *    <my-custom-button bg="blue.400">
 *      Papaya button goes blue
 *    </my-custom-button>
 *    ```
 */
declare const chakra: typeof ___chakra___ & {
    button: ChakraFactoryComponent<{}>;
    input: ChakraFactoryComponent<{}>;
    textarea: ChakraFactoryComponent<{}>;
    select: ChakraFactoryComponent<{}>;
    a: ChakraFactoryComponent<{}>;
    b: ChakraFactoryComponent<{}>;
    article: ChakraFactoryComponent<{}>;
    aside: ChakraFactoryComponent<{}>;
    blockquote: ChakraFactoryComponent<{}>;
    caption: ChakraFactoryComponent<{}>;
    cite: ChakraFactoryComponent<{}>;
    circle: ChakraFactoryComponent<{}>;
    code: ChakraFactoryComponent<{}>;
    dd: ChakraFactoryComponent<{}>;
    div: ChakraFactoryComponent<{}>;
    dl: ChakraFactoryComponent<{}>;
    dt: ChakraFactoryComponent<{}>;
    fieldset: ChakraFactoryComponent<{}>;
    figcaption: ChakraFactoryComponent<{}>;
    figure: ChakraFactoryComponent<{}>;
    footer: ChakraFactoryComponent<{}>;
    form: ChakraFactoryComponent<{}>;
    h1: ChakraFactoryComponent<{}>;
    h2: ChakraFactoryComponent<{}>;
    h3: ChakraFactoryComponent<{}>;
    h4: ChakraFactoryComponent<{}>;
    h5: ChakraFactoryComponent<{}>;
    h6: ChakraFactoryComponent<{}>;
    header: ChakraFactoryComponent<{}>;
    hr: ChakraFactoryComponent<{}>;
    iframe: ChakraFactoryComponent<{}>;
    img: ChakraFactoryComponent<{}>;
    kbd: ChakraFactoryComponent<{}>;
    label: ChakraFactoryComponent<{}>;
    li: ChakraFactoryComponent<{}>;
    mark: ChakraFactoryComponent<{}>;
    main: ChakraFactoryComponent<{}>;
    nav: ChakraFactoryComponent<{}>;
    ol: ChakraFactoryComponent<{}>;
    p: ChakraFactoryComponent<{}>;
    path: ChakraFactoryComponent<{}>;
    pre: ChakraFactoryComponent<{}>;
    q: ChakraFactoryComponent<{}>;
    rect: ChakraFactoryComponent<{}>;
    s: ChakraFactoryComponent<{}>;
    svg: ChakraFactoryComponent<{}>;
    section: ChakraFactoryComponent<{}>;
    strong: ChakraFactoryComponent<{}>;
    small: ChakraFactoryComponent<{}>;
    span: ChakraFactoryComponent<{}>;
    sub: ChakraFactoryComponent<{}>;
    sup: ChakraFactoryComponent<{}>;
    table: ChakraFactoryComponent<{}>;
    tbody: ChakraFactoryComponent<{}>;
    td: ChakraFactoryComponent<{}>;
    tfoot: ChakraFactoryComponent<{}>;
    th: ChakraFactoryComponent<{}>;
    thead: ChakraFactoryComponent<{}>;
    tr: ChakraFactoryComponent<{}>;
    ul: ChakraFactoryComponent<{}>;
};
type EventHandler = (...args: any[]) => void;
declare module "vue" {
    interface ComponentCustomProps extends ChakraProps, StyleResolverProps, HTMLAttributes {
        id?: string;
        role?: string;
        tabindex?: number | string;
        value?: unknown;
        viewBox?: unknown;
        src?: unknown;
        srcset?: unknown;
        crossOrigin?: unknown;
        srcSet?: unknown;
        loading?: unknown;
        alt?: unknown;
        referrerPolicy?: unknown;
        focusable?: unknown;
        type?: unknown;
        disabled?: unknown;
        href?: unknown;
        onBlur?: EventHandler;
        onOpen?: EventHandler;
        onEdit?: EventHandler;
        onLoad?: EventHandler;
        onClose?: EventHandler;
        onFocus?: EventHandler;
        onInput?: EventHandler;
        onClick?: EventHandler;
        onPress?: EventHandler;
        onCancel?: EventHandler;
        onChange?: EventHandler;
        onDelete?: EventHandler;
        onScroll?: EventHandler;
        onSubmit?: EventHandler;
        onSelect?: EventHandler;
        onConfirm?: EventHandler;
        onPreview?: EventHandler;
        onKeypress?: EventHandler;
        onTouchend?: EventHandler;
        onTouchmove?: EventHandler;
        onTouchstart?: EventHandler;
        onTouchcancel?: EventHandler;
        onMouseenter?: EventHandler;
        onMouseleave?: EventHandler;
        onMousemove?: EventHandler;
        onKeydown?: EventHandler;
        onKeyup?: EventHandler;
        onDeselect?: EventHandler;
        onClear?: EventHandler;
    }
}
interface GetStyleObject {
    (options: {
        baseStyle?: SystemStyleObject | ((props: StyleResolverProps) => SystemStyleObject);
    }): FunctionInterpolation<StyleResolverProps>;
}
declare const toCSSObject: GetStyleObject;
interface StyledOptions extends StyleResolverOptions {
    label?: string;
    baseStyle?: SystemStyleObject | ((props: StyleResolverProps) => SystemStyleObject);
}
declare function styled<T extends As, P = {}>(component: T, options: StyledOptions): DefineComponent<{}, () => JSX.Element, {}, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, EmitsOptions, string, VNodeProps & AllowedComponentProps & ComponentCustomProps, Readonly<ExtractPropTypes<{}>>, {}>;
type ChakraComponent<P = ChakraProps> = ComponentWithProps<As & P>;
type HTMLChakraComponents<P> = {
    [Tag in DOMElements]: ChakraComponent<P>;
};
declare const _chakra: {
    button: ChakraFactoryComponent<{}>;
    input: ChakraFactoryComponent<{}>;
    textarea: ChakraFactoryComponent<{}>;
    select: ChakraFactoryComponent<{}>;
    a: ChakraFactoryComponent<{}>;
    b: ChakraFactoryComponent<{}>;
    article: ChakraFactoryComponent<{}>;
    aside: ChakraFactoryComponent<{}>;
    blockquote: ChakraFactoryComponent<{}>;
    caption: ChakraFactoryComponent<{}>;
    cite: ChakraFactoryComponent<{}>;
    circle: ChakraFactoryComponent<{}>;
    code: ChakraFactoryComponent<{}>;
    dd: ChakraFactoryComponent<{}>;
    div: ChakraFactoryComponent<{}>;
    dl: ChakraFactoryComponent<{}>;
    dt: ChakraFactoryComponent<{}>;
    fieldset: ChakraFactoryComponent<{}>;
    figcaption: ChakraFactoryComponent<{}>;
    figure: ChakraFactoryComponent<{}>;
    footer: ChakraFactoryComponent<{}>;
    form: ChakraFactoryComponent<{}>;
    h1: ChakraFactoryComponent<{}>;
    h2: ChakraFactoryComponent<{}>;
    h3: ChakraFactoryComponent<{}>;
    h4: ChakraFactoryComponent<{}>;
    h5: ChakraFactoryComponent<{}>;
    h6: ChakraFactoryComponent<{}>;
    header: ChakraFactoryComponent<{}>;
    hr: ChakraFactoryComponent<{}>;
    iframe: ChakraFactoryComponent<{}>;
    img: ChakraFactoryComponent<{}>;
    kbd: ChakraFactoryComponent<{}>;
    label: ChakraFactoryComponent<{}>;
    li: ChakraFactoryComponent<{}>;
    mark: ChakraFactoryComponent<{}>;
    main: ChakraFactoryComponent<{}>;
    nav: ChakraFactoryComponent<{}>;
    ol: ChakraFactoryComponent<{}>;
    p: ChakraFactoryComponent<{}>;
    path: ChakraFactoryComponent<{}>;
    pre: ChakraFactoryComponent<{}>;
    q: ChakraFactoryComponent<{}>;
    rect: ChakraFactoryComponent<{}>;
    s: ChakraFactoryComponent<{}>;
    svg: ChakraFactoryComponent<{}>;
    section: ChakraFactoryComponent<{}>;
    strong: ChakraFactoryComponent<{}>;
    small: ChakraFactoryComponent<{}>;
    span: ChakraFactoryComponent<{}>;
    sub: ChakraFactoryComponent<{}>;
    sup: ChakraFactoryComponent<{}>;
    table: ChakraFactoryComponent<{}>;
    tbody: ChakraFactoryComponent<{}>;
    td: ChakraFactoryComponent<{}>;
    tfoot: ChakraFactoryComponent<{}>;
    th: ChakraFactoryComponent<{}>;
    thead: ChakraFactoryComponent<{}>;
    tr: ChakraFactoryComponent<{}>;
    ul: ChakraFactoryComponent<{}>;
} & ((tag: ChakraTagOrComponent, options?: StyleResolverOptions & UserProvidedProps) => ChakraFactoryComponent) & HTMLChakraComponents<ChakraProps>;
declare const resolveStyles: (resolvers?: StyleResolverOptions) => CSSObject;
type ChakraFactoryProps = ChakraProps & StyleResolverProps;
/**
 * @example
 * h(chakra(RouterLink, { to: 'https://vue.chakra-ui.com' }), {}, slots)
 */
type UserProvidedProps = {
    [key: string]: any;
};

export { As as A, BaseStyleResolverProps as B, ComponentWithProps2 as C, HTMLChakraProps as H, PropsOf as P, StyleResolverProps as S, Tag as T, _chakra as _, AsPolymorphicProp as a, ChakraBaseComponentProps as b, chakra as c, ChakraComponent as d, ChakraFactoryComponent as e, ChakraFactoryProps as f, ChakraTagOrComponent as g, ComponentWithProps as h, HTMLChakraComponents as i, ChakraProps as j, ___chakra___ as k, resolveStyles as r, styled as s, toCSSObject as t };
